@@ -194,7 +194,7 @@ long long serverPopcount(void *s, long count) {
 #ifdef HAVE_AVX2
     /* If length of s >= 256 bits and the CPU supports AVX2,
      * we prefer to use the SIMD version */
-    if (count >= 32) {
+    if (count >= 32 && __builtin_cpu_supports("avx2")) {
         return popcountAVX2(s, count);
     }
 #endif
