@@ -32,7 +32,11 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  */
-
+/*
+ * Copyright (c) Valkey Contributors
+ * All rights reserved.
+ * SPDX-License-Identifier: BSD-3-Clause
+ */
 #include "fmacros.h"
 
 #include <stddef.h>
@@ -576,7 +580,7 @@ dictEntry *dictAddRaw(dict *d, void *key, dictEntry **existing) {
     if (!position) return NULL;
 
     /* Dup the key if necessary. */
-    if (d->type->keyDup) key = d->type->keyDup(d, key);
+    if (d->type->keyDup) key = d->type->keyDup(key);
 
     return dictInsertAtPosition(d, key, position);
 }
@@ -640,7 +644,7 @@ int dictReplace(dict *d, void *key, void *val) {
      * reverse. */
     void *oldval = dictGetVal(existing);
     dictSetVal(d, existing, val);
-    if (d->type->valDestructor) d->type->valDestructor(d, oldval);
+    if (d->type->valDestructor) d->type->valDestructor(oldval);
     return 0;
 }
 
@@ -742,6 +746,18 @@ dictEntry *dictUnlink(dict *d, const void *key) {
     return dictGenericDelete(d, key, 1);
 }
 
+inline static void dictFreeKey(dict *d, dictEntry *entry) {
+    if (d->type->keyDestructor) {
+        d->type->keyDestructor(dictGetKey(entry));
+    }
+}
+
+inline static void dictFreeVal(dict *d, dictEntry *entry) {
+    if (d->type->valDestructor) {
+        d->type->valDestructor(dictGetVal(entry));
+    }
+}
+
 /* You need to call this function to really free the entry after a call
  * to dictUnlink(). It's safe to call this function with 'he' = NULL. */
 void dictFreeUnlinkedEntry(dict *d, dictEntry *he) {
@@ -885,41 +901,41 @@ void dictTwoPhaseUnlinkFree(dict *d, dictEntry *he, dictEntry **plink, int table
 }
 
 /* In the macros below, `de` stands for dict entry. */
-#define DICT_SET_VALUE(de, field, val)                                                                                 \
-    {                                                                                                                  \
-        if (entryIsNormal(de)) {                                                                                       \
-            dictEntryNormal *_de = decodeEntryNormal(de);                                                              \
-            _de->field = val;                                                                                          \
-        } else if (entryIsEmbedded(de)) {                                                                              \
-            dictEntryEmbedded *_de = decodeEntryEmbedded(de);                                                          \
-            _de->field = val;                                                                                          \
-        } else {                                                                                                       \
-            panic("Entry type not supported");                                                                         \
-        }                                                                                                              \
+#define DICT_SET_VALUE(de, field, val)                        \
+    {                                                         \
+        if (entryIsNormal(de)) {                              \
+            dictEntryNormal *_de = decodeEntryNormal(de);     \
+            _de->field = val;                                 \
+        } else if (entryIsEmbedded(de)) {                     \
+            dictEntryEmbedded *_de = decodeEntryEmbedded(de); \
+            _de->field = val;                                 \
+        } else {                                              \
+            panic("Entry type not supported");                \
+        }                                                     \
     }
-#define DICT_INCR_VALUE(de, field, val)                                                                                \
-    {                                                                                                                  \
-        if (entryIsNormal(de)) {                                                                                       \
-            dictEntryNormal *_de = decodeEntryNormal(de);                                                              \
-            _de->field += val;                                                                                         \
-        } else if (entryIsEmbedded(de)) {                                                                              \
-            dictEntryEmbedded *_de = decodeEntryEmbedded(de);                                                          \
-            _de->field += val;                                                                                         \
-        } else {                                                                                                       \
-            panic("Entry type not supported");                                                                         \
-        }                                                                                                              \
+#define DICT_INCR_VALUE(de, field, val)                       \
+    {                                                         \
+        if (entryIsNormal(de)) {                              \
+            dictEntryNormal *_de = decodeEntryNormal(de);     \
+            _de->field += val;                                \
+        } else if (entryIsEmbedded(de)) {                     \
+            dictEntryEmbedded *_de = decodeEntryEmbedded(de); \
+            _de->field += val;                                \
+        } else {                                              \
+            panic("Entry type not supported");                \
+        }                                                     \
     }
-#define DICT_GET_VALUE(de, field)                                                                                      \
-    (entryIsNormal(de) ? decodeEntryNormal(de)->field                                                                  \
-                       : (entryIsEmbedded(de) ? decodeEntryEmbedded(de)->field                                         \
+#define DICT_GET_VALUE(de, field)                                              \
+    (entryIsNormal(de) ? decodeEntryNormal(de)->field                          \
+                       : (entryIsEmbedded(de) ? decodeEntryEmbedded(de)->field \
                                               : (panic("Entry type not supported"), ((dictEntryNormal *)de)->field)))
-#define DICT_GET_VALUE_PTR(de, field)                                                                                  \
-    (entryIsNormal(de)                                                                                                 \
-         ? &decodeEntryNormal(de)->field                                                                               \
+#define DICT_GET_VALUE_PTR(de, field)    \
+    (entryIsNormal(de)                   \
+         ? &decodeEntryNormal(de)->field \
          : (entryIsEmbedded(de) ? &decodeEntryEmbedded(de)->field : (panic("Entry type not supported"), NULL)))
 
 void dictSetKey(dict *d, dictEntry *de, void *key) {
-    void *k = d->type->keyDup ? d->type->keyDup(d, key) : key;
+    void *k = d->type->keyDup ? d->type->keyDup(key) : key;
     if (entryIsNormal(de)) {
         dictEntryNormal *_de = decodeEntryNormal(de);
         _de->key = k;
@@ -1309,7 +1325,7 @@ end:
 
 /* Reallocate the dictEntry, key and value allocations in a bucket using the
  * provided allocation functions in order to defrag them. */
-static void dictDefragBucket(dictEntry **bucketref, dictDefragFunctions *defragfns, void *privdata) {
+static void dictDefragBucket(dictEntry **bucketref, const dictDefragFunctions *defragfns, void *privdata) {
     dictDefragAllocFunction *defragalloc = defragfns->defragAlloc;
     dictDefragAllocFunction *defragkey = defragfns->defragKey;
     dictDefragAllocFunction *defragval = defragfns->defragVal;
@@ -1487,7 +1503,7 @@ unsigned long dictScan(dict *d, unsigned long v, dictScanFunction *fn, void *pri
  * where NULL means that no reallocation happened and the old memory is still
  * valid. */
 unsigned long
-dictScanDefrag(dict *d, unsigned long v, dictScanFunction *fn, dictDefragFunctions *defragfns, void *privdata) {
+dictScanDefrag(dict *d, unsigned long v, dictScanFunction *fn, const dictDefragFunctions *defragfns, void *privdata) {
     int htidx0, htidx1;
     const dictEntry *de, *next;
     unsigned long m0, m1;
