@@ -704,8 +704,8 @@ int test_random_entry(int argc, char **argv, int flags) {
     hashtable *ht = hashtableCreate(&type);
 
     /* Populate */
-    unsigned times_picked[count];
-    memset(times_picked, 0, sizeof(times_picked));
+    unsigned *times_picked = zmalloc(sizeof(unsigned) * count);
+    memset(times_picked, 0, sizeof(unsigned) * count);
     for (size_t j = 0; j < count; j++) {
         TEST_ASSERT(hashtableAdd(ht, times_picked + j));
     }
@@ -763,6 +763,8 @@ int test_random_entry(int argc, char **argv, int flags) {
         p10percent += (dev >= -0.1 * expected && dev <= 0.1 * expected);
     }
 
+    zfree(times_picked);
+
     printf("Random entry fairness test\n");
     printf("  Pick one of %zu entries, %ld times.\n", count, num_rounds);
     printf("  Expecting each entry to be picked %.2lf times, std dev %.3lf.\n", expected, std_dev);
@@ -879,6 +881,69 @@ int test_random_entry_with_long_chain(int argc, char **argv, int flags) {
     return 0;
 }
 
+static void deleteScanFn(void *privdata, void *entry) {
+    hashtable *ht = privdata;
+    hashtableDelete(ht, entry);
+}
+
+int test_random_entry_sparse_table(int argc, char **argv, int flags) {
+    UNUSED(argc);
+    UNUSED(argv);
+    UNUSED(flags);
+    randomSeed();
+
+    size_t count = (flags & UNIT_TEST_LARGE_MEMORY) ? 100000000 : 1000000;
+    long num_rounds = (flags & UNIT_TEST_ACCURATE) ? 256 * 1024 : 1024;
+
+    /* A set of pointers */
+    hashtableType type = {0};
+    hashtable *ht = hashtableCreate(&type);
+    monotime timer;
+    monotonicInit();
+
+    /* Populate */
+    unsigned *values = zmalloc(sizeof(unsigned) * count);
+    for (size_t j = 0; j < count; j++) {
+        TEST_ASSERT(hashtableAdd(ht, &values[j]));
+    }
+
+    /* Pick random elements */
+    elapsedStart(&timer);
+    for (long i = 0; i < num_rounds; i++) {
+        void *entry;
+        TEST_ASSERT(hashtableFairRandomEntry(ht, &entry));
+        unsigned *picked = entry;
+        TEST_ASSERT(picked >= values && picked < values + count);
+    }
+    uint64_t us0 = elapsedUs(timer);
+    printf("Fair random, filled hashtable, avg time: %.3lfµs\n", (double)us0 / num_rounds);
+
+    size_t cursor = random();
+
+    for (int n = 2; n <= 8; n *= 2) {
+        /* Scan and delete until only 1/n of the values remain. */
+        while (hashtableSize(ht) > count / n) {
+            cursor = hashtableScan(ht, cursor, deleteScanFn, ht);
+        }
+
+        /* Pick random elements. */
+        elapsedStart(&timer);
+        for (long i = 0; i < num_rounds; i++) {
+            void *entry;
+            TEST_ASSERT(hashtableFairRandomEntry(ht, &entry));
+            unsigned *picked = entry;
+            TEST_ASSERT(picked >= values && picked < values + count);
+        }
+        uint64_t us = elapsedUs(timer);
+        printf("Fair random, 1/%d filled hashtable, avg time: %.3lfµs\n", n, (double)us / num_rounds);
+        /* Allow max 10 times slower than in a dense table. */
+        TEST_ASSERT(us <= us0 * 10);
+    }
+    hashtableRelease(ht);
+    zfree(values);
+    return 0;
+}
+
 int test_all_memory_freed(int argc, char **argv, int flags) {
     UNUSED(argc);
     UNUSED(argv);
