@@ -1,8 +1,8 @@
-tags {external:skip cluster} {
+tags {external:skip cluster singledb} {
+    set old_singledb $::singledb
+    set ::singledb 1
     # Start a cluster with a divergent shard ID configuration
     test "divergent cluster shardid conflict" {
-        set old_singledb $::singledb
-        set ::singledb 1
         for {set i 1} {$i <= 4} {incr i} {
             if {$::verbose} { puts "Testing for tests/assets/divergent-shard-$i.conf"; flush stdout;}
             exec cp -f tests/assets/divergent-shard-$i.conf tests/tmp/nodes.conf.divergent
@@ -12,6 +12,6 @@ tags {external:skip cluster} {
                 assert_equal $count 2 "Expect shard ID to be present twice in the configuration file"
             }
         }
-        set ::singledb $old_singledb
     }
+    set ::singledb $old_singledb
 }
