@@ -4,7 +4,7 @@ start_server {tags {"maxmemory" "external:skip"}} {
     set server_pid [s process_id]
 
     proc init_test {client_eviction} {
-        r flushdb
+        r flushdb sync
 
         set prev_maxmemory_clients [r config get maxmemory-clients]
         if $client_eviction {
@@ -56,7 +56,7 @@ start_server {tags {"maxmemory" "external:skip"}} {
             init_test $client_eviction
 
             for {set j 0} {$j < 20} {incr j} {
-                set rr [redis_deferring_client]
+                set rr [valkey_deferring_client]
                 lappend clients $rr
             }
             
@@ -85,7 +85,7 @@ start_server {tags {"maxmemory" "external:skip"}} {
             init_test $client_eviction
             
             for {set j 0} {$j < 30} {incr j} {
-                set rr [redis_deferring_client]
+                set rr [valkey_deferring_client]
                 lappend clients $rr
             }
 
@@ -98,6 +98,7 @@ start_server {tags {"maxmemory" "external:skip"}} {
                         $rr write "\r\n"
                         $rr flush
                     }
+                    after 100; # give the server some time to process the input buffer - this was added to make sure the test pass with io-threads active.
                 }]} {
                     lremove clients $rr
                 }
@@ -114,7 +115,7 @@ start_server {tags {"maxmemory" "external:skip"}} {
             init_test $client_eviction
 
             for {set j 0} {$j < 20} {incr j} {
-                set rr [redis_client]
+                set rr [valkey_client]
                 lappend clients $rr
             }
 
@@ -144,30 +145,6 @@ start_server {tags {"maxmemory" "external:skip"}} {
 }
 
 start_server {tags {"maxmemory external:skip"}} {
-    test "Without maxmemory small integers are shared" {
-        r config set maxmemory 0
-        r set a 1
-        assert_refcount_morethan a 1
-    }
-
-    test "With maxmemory and non-LRU policy integers are still shared" {
-        r config set maxmemory 1073741824
-        r config set maxmemory-policy allkeys-random
-        r set a 1
-        assert_refcount_morethan a 1
-    }
-
-    test "With maxmemory and LRU policy integers are not shared" {
-        r config set maxmemory 1073741824
-        r config set maxmemory-policy allkeys-lru
-        r set a 1
-        r config set maxmemory-policy volatile-lru
-        r set b 1
-        assert_refcount 1 a
-        assert_refcount 1 b
-        r config set maxmemory 0
-    }
-
     foreach policy {
         allkeys-random allkeys-lru allkeys-lfu volatile-lru volatile-lfu volatile-random volatile-ttl
     } {
@@ -225,7 +202,7 @@ start_server {tags {"maxmemory external:skip"}} {
             }
             # If we add the same number of keys already added again and
             # the policy is allkeys-* we should still be under the limit.
-            # Otherwise we should see an error reported by Redis.
+            # Otherwise we should see an error reported by Server.
             set err 0
             for {set j 0} {$j < $numkeys} {incr j} {
                 if {[catch {r set [randomKey] x} e]} {
@@ -249,10 +226,10 @@ start_server {tags {"maxmemory external:skip"}} {
             # make sure to start with a blank instance
             r flushall
             # Get the current memory limit and calculate a new limit.
-            # We just add 100k to the current memory size so that it is
+            # We just add 400KiB to the current memory size so that it is
             # fast for us to reach that limit.
             set used [s used_memory]
-            set limit [expr {$used+100*1024}]
+            set limit [expr {$used+400*1024}]
             r config set maxmemory $limit
             r config set maxmemory-policy $policy
             # Now add keys until the limit is almost reached.
@@ -272,7 +249,7 @@ start_server {tags {"maxmemory external:skip"}} {
                 incr numkeys
             }
             # Now we add the same number of volatile keys already added.
-            # We expect Redis to evict only volatile keys in order to make
+            # We expect the server to evict only volatile keys in order to make
             # space.
             set err 0
             for {set j 0} {$j < $numkeys} {incr j} {
@@ -349,12 +326,12 @@ proc test_slave_buffers {test_name cmd_count payload_len limit_memory pipeline}
             }
 
             # put the slave to sleep
-            set rd_slave [redis_deferring_client]
+            set rd_slave [valkey_deferring_client]
             pause_process $slave_pid
 
             # send some 10mb worth of commands that don't increase the memory usage
             if {$pipeline == 1} {
-                set rd_master [redis_deferring_client -1]
+                set rd_master [valkey_deferring_client -1]
                 for {set k 0} {$k < $cmd_count} {incr k} {
                     $rd_master setrange key:0 0 [string repeat A $payload_len]
                 }
@@ -406,7 +383,7 @@ proc test_slave_buffers {test_name cmd_count payload_len limit_memory pipeline}
 
 # test that slave buffer are counted correctly
 # we wanna use many small commands, and we don't wanna wait long
-# so we need to use a pipeline (redis_deferring_client)
+# so we need to use a pipeline (valkey_deferring_client)
 # that may cause query buffer to fill and induce eviction, so we disable it
 test_slave_buffers {slave buffer are counted correctly} 1000000 10 0 1
 
@@ -419,33 +396,52 @@ start_server {tags {"maxmemory external:skip"}} {
         r config set latency-tracking no
         r config set maxmemory 0
         r config set maxmemory-policy allkeys-random
+        set dbnum [expr {$::singledb ? 0 : 9}]
 
-        # Next rehash size is 8192, that will eat 64k memory
-        populate 4096 "" 1
+        # Populate some, then check table size and populate more up to one less
+        # than the soft maximum fill factor. Adding some more elements after
+        # this does not trigger rehashing, because rehashing would eat some
+        # kilobytes of memory.
+        populate 2000 a 1
+        set table_size [main_hash_table_size]
+        populate [main_hash_table_keys_before_rehashing_starts] b 1
 
+        # Now we are close to resizing. Check that rehashing didn't start.
+        assert_equal $table_size [main_hash_table_size]
+        assert_no_match "*Hash table 1 stats*" [r debug htstats $dbnum]
+
+        set dbsize_before [r dbsize]
         set used [s used_memory]
         set limit [expr {$used + 10*1024}]
         r config set maxmemory $limit
+
+        # Adding a key to meet the 1:1 radio.
+        r set k0 v0
+        # The table has reached the soft max fill factor.
+        # It can be resized in tryResizeHashTables in cron,
+        # or we add a key to let it check whether it can be resized.
         r set k1 v1
         # Next writing command will trigger evicting some keys if last
         # command trigger DB dict rehash
         r set k2 v2
-        # There must be 4098 keys because redis doesn't evict keys.
-        r dbsize
-    } {4098}
+        # There must be three more keys because the server doesn't evict keys.
+        assert_equal [r dbsize] [expr {$dbsize_before + 3}]
+    }
 }
 
-start_server {tags {"maxmemory external:skip"}} {
+# Skip the following test when running with IO threads
+# With IO threads, we asynchronously write to tracking clients.
+# This invalidates the assumption that their output buffers will be free within the same event loop.
+start_server {tags {"maxmemory external:skip io-threads:skip"}} {
     test {client tracking don't cause eviction feedback loop} {
         r config set latency-tracking no
         r config set maxmemory 0
         r config set maxmemory-policy allkeys-lru
         r config set maxmemory-eviction-tenacity 100
-
         # 10 clients listening on tracking messages
         set clients {}
         for {set j 0} {$j < 10} {incr j} {
-            lappend clients [redis_deferring_client]
+            lappend clients [valkey_deferring_client]
         }
         foreach rd $clients {
             $rd HELLO 3
@@ -527,9 +523,9 @@ start_server {tags {"maxmemory" "external:skip"}} {
             {set asdf1 1}
             {set asdf2 2}
             {set asdf3 3}
-            {del asdf*}
-            {del asdf*}
-            {del asdf*}
+            {unlink asdf*}
+            {unlink asdf*}
+            {unlink asdf*}
             {set asdf4 4}
         }
         close_replication_stream $repl
@@ -563,7 +559,7 @@ start_server {tags {"maxmemory" "external:skip"}} {
             {incr x}
             {incr x}
             {exec}
-            {del x}
+            {unlink x}
         }
         close_replication_stream $repl
 
@@ -588,3 +584,21 @@ start_server {tags {"maxmemory" "external:skip"}} {
         assert {[r object freq foo] == 5}
     }
 }
+
+start_server {tags {"maxmemory" "external:skip"}} {
+    test {Import mode should forbid eviction} {
+        r set key val
+        r config set import-mode yes
+        assert_equal [r client import-source on] {OK}  
+        r config set maxmemory-policy allkeys-lru
+        r config set maxmemory 1      
+
+        assert_equal [r dbsize] {1}
+        assert_error {OOM command not allowed*} {r set key1 val1}
+
+        assert_equal [r client import-source off] {OK}  
+        r config set import-mode no
+
+        assert_equal [r dbsize] {0}
+    }
+}
