@@ -104,15 +104,22 @@ start_server {tags {"modules acl"}} {
         assert_equal [r acl DRYRUN j2 aclcheck.module.command.aclcategories.read.only.category] OK
     }
 
+    test {Unload the module - aclcheck} {
+        assert_equal {OK} [r module unload aclcheck]
+    }
+}
+
+start_server {tags {"modules acl"}} {
     test {test existing users to have access to module commands loaded on runtime} {
-        assert_equal [r module unload aclcheck] OK
         r acl SETUSER j3 on >password -@all +@WRITE
         assert_equal [r module load $testmodule] OK
         assert_equal [r acl DRYRUN j3 aclcheck.module.command.aclcategories.write] OK
+        assert_equal {OK} [r module unload aclcheck]
     }
+}
 
+start_server {tags {"modules acl"}} {
     test {test existing users without permissions, do not have access to module commands loaded on runtime.} {
-        assert_equal [r module unload aclcheck] OK
         r acl SETUSER j4 on >password -@all +@READ
         r acl SETUSER j5 on >password -@all +@WRITE
         assert_equal [r module load $testmodule] OK
@@ -131,44 +138,25 @@ start_server {tags {"modules acl"}} {
         assert_equal {User j7 has no permissions to run the 'aclcheck.module.command.aclcategories.write.function.read.category' command} $e
     }
 
-    test "Unload the module - aclcheck" {
+    test {test if foocategory acl categories is added} {
+        r acl SETUSER j8 on >password -@all +@foocategory
+        assert_equal [r acl DRYRUN j8 aclcheck.module.command.test.add.new.aclcategories] OK
+    }
+
+    test {test if ACL CAT output for the new category is correct} {
+        assert_equal [r ACL CAT foocategory] aclcheck.module.command.test.add.new.aclcategories
+    }
+
+    test {test permission compaction and simplification for categories added by a module} {
+        r acl SETUSER j9 on >password -@all +@foocategory -@foocategory
+        catch {r ACL GETUSER j9} res
+        assert_equal {-@all -@foocategory} [lindex $res 5]
         assert_equal {OK} [r module unload aclcheck]
     }
 }
 
-set subcommandsmodule [file normalize tests/modules/subcommands.so]
 start_server {tags {"modules acl"}} {
-    r module load $subcommandsmodule
-
-    test {Module unload blocked by ACL subcommand rule} {
-        r ACL SETUSER subcmduser on nopass +subcommands.sub|get_fullname
-        catch {r module unload subcommands} e
-        assert_match {*one or more ACL users reference commands from this module*} $e
-        r ACL DELUSER subcmduser
-    }
-
-    test {Module unload blocked by ACL base command rule} {
-        r ACL SETUSER basecmduser on nopass +subcommands.parent_get_fullname
-        catch {r module unload subcommands} e
-        assert_match {*one or more ACL users reference commands from this module*} $e
-        r ACL DELUSER basecmduser
-    }
-
-    test {Module unload blocked by ACL deny rule} {
-        r ACL SETUSER denycmduser on nopass -subcommands.parent_get_fullname
-        catch {r module unload subcommands} e
-        assert_match {*one or more ACL users reference commands from this module*} $e
-        r ACL DELUSER denycmduser
-    }
-
-    test {Module unload blocked by ACL selector rule} {
-        r ACL SETUSER selcmduser on nopass (+subcommands.parent_get_fullname)
-        catch {r module unload subcommands} e
-        assert_match {*one or more ACL users reference commands from this module*} $e
-        r ACL DELUSER selcmduser
-    }
-
-    test {Unload the module - subcommands} {
-        assert_equal {OK} [r module unload subcommands]
+    test {test module load fails if exceeds the maximum number of adding acl categories} {
+        assert_error {ERR Error loading the extension. Please check the server logs.} {r module load $testmodule 1}
     }
 }
