@@ -34,7 +34,29 @@ start_server {tags {"modules"}} {
         }
     }
 
-    test "Unload the module - test" {
+    test "Busy module name" {
+        assert_error {ERR Error loading the extension. Please check the server logs.} {r module load $testmodule}
+        verify_log_message 0 "*Module name is busy*" 0
+    }
+
+    test "test latency" {
+        r config set latency-monitor-threshold 0
+        r latency reset
+        r test.latency 0
+        r test.latency 1
+        assert_equal {} [r latency latest]
+        assert_equal {} [r latency history test]
+
+        r config set latency-monitor-threshold 1
+        r test.latency 0
+        assert_equal 0 [llength [r latency history test]]
+        r test.latency 1
+        assert_match {*test * 1 1*} [r latency latest]
+        r test.latency 2
+        assert_match {*test * 2 2*} [r latency latest]
+    }
+
+    test "Unload the module - basics" {
         assert_equal {OK} [r module unload test]
     }
 }
