@@ -363,6 +363,11 @@ start_server {tags {"hash"}} {
         assert_error "WRONGTYPE Operation against a key*" {r hvals wrongtype}
         assert_error "WRONGTYPE Operation against a key*" {r hkeys wrongtype}
         assert_error "WRONGTYPE Operation against a key*" {r hexists wrongtype field1}
+        assert_error "WRONGTYPE Operation against a key*" {r hset wrongtype field1 val1}
+        assert_error "WRONGTYPE Operation against a key*" {r hmset wrongtype field1 val1 field2 val2}
+        assert_error "WRONGTYPE Operation against a key*" {r hsetnx wrongtype field1 val1}
+        assert_error "WRONGTYPE Operation against a key*" {r hlen wrongtype}
+        assert_error "WRONGTYPE Operation against a key*" {r hscan wrongtype 0}
     }
 
     test {HMGET - small hash} {
@@ -429,6 +434,11 @@ start_server {tags {"hash"}} {
         lsort [r hgetall bighash]
     } [lsort [array get bighash]]
 
+    test {HGETALL against non-existing key} {
+        r del htest
+        r hgetall htest
+    } {}
+
     test {HDEL and return value} {
         set rv {}
         lappend rv [r hdel smallhash nokey]
@@ -473,6 +483,7 @@ start_server {tags {"hash"}} {
     } {1 0 1 0}
 
     test {Is a ziplist encoded Hash promoted on big payload?} {
+        r config set hash-max-listpack-value 64
         r hset smallhash foo [string repeat a 1024]
         r debug object smallhash
     } {*hashtable*} {needs:debug}
@@ -482,6 +493,13 @@ start_server {tags {"hash"}} {
         list [r hincrby htest foo 2]
     } {2}
 
+    test {HINCRBY HINCRBYFLOAT against non-integer increment value} {
+        r del incrhash
+        r hset incrhash field 5
+        assert_error "*value is not an integer*" {r hincrby incrhash field v}
+        assert_error "*value is not a*" {r hincrbyfloat incrhash field v}
+    }
+
     test {HINCRBY against non existing hash key} {
         set rv {}
         r hdel smallhash tmp
